/* MIT License

Copyright (c) 2017 Adarsh Yoga and Santosh Nagarakatte, Rutgers University

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. */

#ifndef TASK_PROFILER_H
#define TASK_PROFILER_H

#include <string>
#include <unordered_map>

#include "AFTaskGraph.H"
#include "WorkSpanData.H"

class Task_Profiler {
private:
  AFTaskGraph* taskGraph;
  std::unordered_map<size_t, struct CallSiteData*> regionMap;
  std::unordered_map<size_t, struct WorkSpanData>* workSpanMap;

  bool recursiveCall(struct AFTask* node);
  void checkUpdateParentWorkSpan(size_t parent_index, size_t* num_processed, std::unordered_map<size_t, struct WorkSpanData>* workSpanMap);
  void calculateRecurse(struct AFTask* node, std::unordered_map<size_t, struct WorkSpanData>* workSpanMap);
  void calculateWorkSpan(struct AFTask* node);
  bool checkRecursiveCall(struct AFTask* node, size_t call_site);

  /********** INCREASE PARALLELISM OF EACH STATIC REGION **************/
  void report_region_increase_par();
  void checkUpdateParentWorkSpan_region(size_t parent_index, size_t* num_processed, size_t updated_region, unsigned int par_increase);
  void calculateRecurse_region(struct AFTask* node, size_t updated_region, unsigned int par_increase);
  void calculateWorkSpan_region(struct AFTask* node, size_t updated_region, unsigned int par_increase);
  void reset_work_span();
  /***************************************************************/


  /********** REPORT %AGE OF CRITICAL PATH LENGTH **************/
  void merge_critical_call_sites(std::unordered_map<size_t, size_t>*, std::unordered_map<size_t, size_t>*);
  void insert_cs_data(std::unordered_map<size_t, size_t>*, size_t call_site, size_t local_local_work);
  /***************************************************************/

public:
  Task_Profiler();
  size_t GenerateProfile(std::string);
};

#endif
