/* MIT License

Copyright (c) 2017 Adarsh Yoga and Santosh Nagarakatte, Rutgers University

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. */

#ifndef AFTASK_GRAPH_H
#define AFTASK_GRAPH_H

#include <stack>
#include <unordered_map>
#include <fstream>

#include "AFTask.H"
#include "Common.H"
#include "CallSiteData.H"
#include "Task_Logger.H"

extern Task_Logger* taskLogger;

class AFTaskGraph {
protected:
  std::stack<struct AFTask*> cur_dpst_index[NUM_THREADS];
  std::unordered_map<size_t, struct AFTask*> temp_cur_map;
  std::unordered_map<size_t, struct CallSiteData*> callSiteMap;
  size_t last_allocated_node;

  void CaptureSpawnOnly(THREADID threadid, size_t taskId, void* return_address);
  void CaptureSpawnAndWaitForAll(THREADID threadid, size_t taskId, void* return_address);
  void CaptureSpawnAndWait(THREADID threadid, size_t taskId, void* return_address);
  struct AFTask* create_node(THREADID threadid);

public:
  AFTaskGraph();
  void CaptureExecute(THREADID threadid, size_t taskId);
  void CaptureReturn(THREADID threadid);
  void CaptureWaitOnly(THREADID threadid);
  void CaptureWait(THREADID threadid);
  void CaptureSetTaskId(THREADID threadid, size_t taskId, 
			int sp_only, void* return_address, 
			const char* file, int line, bool par_for = false);
  void CaptureSetTaskId(THREADID threadid, size_t taskId, 
			void* return_address, 
			const char* file, int line, bool par_for = false);
  size_t getCurStepParent(THREADID threadid);
  size_t getCurStepId(THREADID threadid);
  struct location getCurStepRegionBegin(THREADID threadid);
  struct location getCurStepRegionEnd(THREADID threadid);
  void dumpCallsiteInfo();
};

#endif
